@@ -7,13 +7,18 @@ of a view. For documentation on how views are represented, see
 :mod:`mmif.serialize.view`.
 """
 
-from typing import Union
+import pathlib
+from urllib.parse import urlparse
+from typing import Union, Dict, List, Type
 from pyrsistent import pmap, pvector
+
 from .model import FreezableMmifObject
 from mmif.vocabulary import ThingTypesBase, DocumentTypesBase
 
 __all__ = ['Annotation', 'AnnotationProperties', 'Document', 'DocumentProperties', 'Text']
 
+JSON_COMPATIBLE_PRIMITIVES: Type = Union[str, int, float, bool, None]
+
 
 class Annotation(FreezableMmifObject):
     """
@@ -22,11 +27,10 @@ class Annotation(FreezableMmifObject):
 
     def __init__(self, anno_obj: Union[bytes, str, dict] = None) -> None:
         self._type: Union[str, ThingTypesBase] = ''
-        if 'properties' not in self.__dict__:  # don't overwrite DocumentProperties on super() call
+        if not hasattr(self, 'properties'):  # don't overwrite DocumentProperties on super() call
             self.properties: AnnotationProperties = AnnotationProperties()
-        self.disallow_additional_properties()
-        if 'properties' not in self._attribute_classes:
             self._attribute_classes = pmap({'properties': AnnotationProperties})
+        self.disallow_additional_properties()
         self._required_attributes = pvector(["_type", "properties"])
         super().__init__(anno_obj)
 
@@ -54,14 +58,30 @@ class Annotation(FreezableMmifObject):
     def id(self, aid: str) -> None:
         self.properties.id = aid
 
-    def add_property(self, name: str, value: str) -> None:
+    def add_property(self, name: str,
+                     value: Union[JSON_COMPATIBLE_PRIMITIVES,
+                                  List[JSON_COMPATIBLE_PRIMITIVES],
+                                  List[List[JSON_COMPATIBLE_PRIMITIVES]]
+                    ]) -> None:
         """
         Adds a property to the annotation's properties.
         :param name: the name of the property
         :param value: the property's desired value
         :return: None
         """
-        self.properties[name] = value
+        json_primitives = lambda x:isinstance(x, JSON_COMPATIBLE_PRIMITIVES.__args__)
+        if json_primitives(value) or (
+                isinstance(value,list)
+                and all(map(json_primitives, value)) or (
+                        all(map(lambda elem: isinstance(elem, list), value))
+                        and map(json_primitives, [subelem for elem in value for subelem in elem])
+                )
+        ):
+            self.properties[name] = value
+        else:
+            raise ValueError("Property values cannot be a complex object. It must be "
+                             "either string, number, boolean, None, or a list of them."
+                             f"(\"{name}\": \"{str(value)}\"")
 
     def is_document(self):
         return self.at_type.endswith("Document")
@@ -98,6 +118,32 @@ class Document(Annotation):
         # but import `View` will break the code due to circular imports
         self._parent_view_id = parent_view_id
 
+    def add_property(self, name: str,
+                     value: Union[JSON_COMPATIBLE_PRIMITIVES,
+                                  List[JSON_COMPATIBLE_PRIMITIVES]]) -> None:
+        if name == "text":
+            self.properties.text = Text(value)
+        elif name == "location":
+            self.location = value
+        else:
+            super().add_property(name, value)
+
+    @property
+    def text_language(self) -> str:
+        return self.properties.text_language
+
+    @text_language.setter
+    def text_language(self, lang_code: str) -> None:
+        self.properties.text_language = lang_code
+
+    @property
+    def text_value(self) -> str:
+        return self.properties.text_value
+
+    @text_value.setter
+    def text_value(self, text_value: str) -> None:
+        self.properties.text_value = text_value
+
     @property
     def location(self) -> str:
         return self.properties.location
@@ -106,6 +152,15 @@ class Document(Annotation):
     def location(self, location: str) -> None:
         self.properties.location = location
 
+    def location_scheme(self) -> str:
+        return self.properties.location_scheme()
+
+    def location_address(self) -> str:
+        return self.properties.location_address()
+
+    def location_path(self) -> str:
+        return self.properties.location_path()
+
 
 class AnnotationProperties(FreezableMmifObject):
     """
@@ -131,11 +186,27 @@ class DocumentProperties(AnnotationProperties):
 
     def __init__(self, mmif_obj: Union[bytes, str, dict] = None) -> None:
         self.mime: str = ''
-        self.location: str = ''
+        # note the trailing underscore here. I wanted to use the name `location`
+        # for @property in this class and `Document` class, so had to use a diff
+        # name for the variable. See `_serialize()` and `_deserialize()` below
+        # to see how this exception is handled
+        self.location_: str = ''
         self.text: Text = Text()
         self._attribute_classes = pmap({'text': Text})
         super().__init__(mmif_obj)
 
+    def _deserialize(self, input_dict: dict) -> None:
+        if "location" in input_dict:
+            self.location = input_dict.pop("location")
+        super()._deserialize(input_dict)
+
+    def _serialize(self, alt_container: Dict = None) -> dict:
+        serialized = super()._serialize()
+        if "location_" in serialized:
+            serialized["location"] = serialized.pop("location_")
+        return serialized
+
+
     @property
     def text_language(self) -> str:
         return self.text.lang
@@ -152,6 +223,31 @@ class DocumentProperties(AnnotationProperties):
     def text_value(self, s: str) -> None:
         self.text.value = s
 
+    @property
+    def location(self) -> str:
+        return self.location_
+
+    @location.setter
+    def location(self, location: str) -> None:
+        parsed_location = urlparse(location)
+        if parsed_location.scheme is None or len(parsed_location.scheme) == 0:
+            self.location_ = pathlib.Path(location).as_uri()
+        else:
+            self.location_ = location
+
+    def location_scheme(self) -> str:
+        return urlparse(self.location).scheme
+
+    def location_address(self) -> str:
+        parsed_location = urlparse(self.location)
+        if len(parsed_location.netloc) == 0:
+            return parsed_location.path
+        else:
+            return "".join((parsed_location.netloc, parsed_location.path))
+
+    def location_path(self) -> str:
+        return urlparse(self.location).path
+
 
 class Text(FreezableMmifObject):
 
