@@ -10,14 +10,17 @@ from datetime import datetime
 from typing import List, Union, Optional, Dict, ClassVar
 
 import jsonschema.validators
+from mmif import DocumentTypes
 from pkg_resources import resource_stream
 
 import mmif
-from pyrsistent import pmap
+from mmif import ThingTypesBase, DocumentTypes
+from pyrsistent import pmap, pvector
 
 from .view import View
 from .annotation import Annotation, Document
 from .model import MmifObject, DataList, FreezableDataList
+from mmif.vocabulary import AnnotationTypes, DocumentTypes
 
 __all__ = ['Mmif']
 
@@ -43,6 +46,7 @@ class Mmif(MmifObject):
             'documents': DocumentsList,
             'views': ViewsList
         })
+        self._required_attributes = pvector(["metadata", "documents", "views"])
         super().__init__(mmif_obj)
         if frozen:
             self.freeze_documents()
@@ -67,7 +71,7 @@ class Mmif(MmifObject):
         schema_res = resource_stream(f'{mmif.__name__}.{mmif._res_pkg}', mmif._schema_res_name)
         schema = json.load(schema_res)
         schema_res.close()
-        if type(json_str) == str:
+        if isinstance(json_str, str):
             json_str = json.loads(json_str)
         jsonschema.validate(json_str, schema)
 
@@ -158,6 +162,20 @@ class Mmif(MmifObject):
         else:
             return []
 
+    def get_documents_by_type(self, doc_type: Union[str, DocumentTypes]) -> List[Document]:
+        """
+        Method to get all documents where the type matches a particular document type, which should be one of the CLAMS document types.
+
+        :param doc_type: the type of documents to search for, must be one of ``Document`` type defined in the CLAMS vocabulary.
+        :return: a list of documents matching the requested type, or an empty list if none found.
+        """
+        docs = []
+        # although only `TextDocument`s are allowed in view:annotations list, this implementation is more future-proof
+        for view in self.views:
+            docs.extend([document for document in view.get_documents() if document.is_type(doc_type)])
+        docs.extend([document for document in self.documents if document.is_type(doc_type)])
+        return docs
+
     def get_documents_by_app(self, app_id: str) -> List[Document]:
         """
         Method to get all documents object queries by its originated app name.
@@ -165,12 +183,11 @@ class Mmif(MmifObject):
         :param app_id: the app name to search for
         :return: a list of documents matching the requested app name, or an empty list if the app not found
         """
-        # TODO (krim @ 9/19/20): what if there are two or more views generated
-        #  by the same app (separately)
+        docs = []
         for view in self.views:
             if view.metadata.app == app_id:
-                return view.get_documents()
-        return []
+                docs.extend(view.get_documents())
+        return docs
 
     def get_documents_by_property(self, prop_key: str, prop_value: str) -> List[Document]:
         """
@@ -184,16 +201,11 @@ class Mmif(MmifObject):
         for view in self.views:
             for doc in view.get_documents():
                 if doc.properties[prop_key] == prop_value:
-                    # TODO (krim @ 9/19/20): we can have a reserved_name in `Document` to store parent view id,
-                    # instead of changing the doc ID
-                    # TODO (krim @ 9/19/20): colon should be stored as a constant with a proper name
-                    if ":" not in doc.id:
-                        doc.id = f"{view.id}:{doc.id}"
                     docs.append(doc)
         docs.extend([document for document in self.documents if document.properties[prop_key] == prop_value])
         return docs
 
-    def get_documents_locations(self, m_type: str) -> List[str]:
+    def get_documents_locations(self, m_type: Union[DocumentTypes, str]) -> List[str]:
         """
         This method returns the file paths of documents of given type.
         Only top-level documents have locations, so we only check them.
@@ -201,9 +213,9 @@ class Mmif(MmifObject):
         :param m_type: the type to search for
         :return: a list of the values of the location fields in the corresponding documents
         """
-        return [document.location for document in self.documents if document.at_type == m_type and len(document.location) > 0]
+        return [document.location for document in self.documents if document.is_type(m_type) and len(document.location) > 0]
 
-    def get_document_location(self, m_type: str) -> str:
+    def get_document_location(self, m_type: Union[DocumentTypes, str]) -> str:
         """
         Method to get the location of *first* document of given type.
 
@@ -244,29 +256,94 @@ class Mmif(MmifObject):
             raise KeyError("{} view not found".format(req_view_id))
         return result
 
-    def get_all_views_contain(self, at_type: str) -> List[View]:
+    def get_alignments(self, at_type1: Union[str, ThingTypesBase], at_type2: Union[str, ThingTypesBase]) -> Dict[str, List[Annotation]]:
         """
-        Returns the list of all views in the MMIF if a given type
-        type is present in that view's 'contains' metadata.
+        Finds views where alignments between two given annotation types occurred.
 
-        :param at_type: the type to check for
+        :return: a dict that keyed by view IDs (str) and has lists of alignment Annotation objects as values.
+        """
+        v_and_a = {}
+        for alignment_view in self.get_all_views_contain(AnnotationTypes.Alignment):
+            alignments = []
+            # TODO (krim @ 11/7/20): maybe Alignment can have metadata on what types are aligned?
+            for alignment in alignment_view.get_annotations(AnnotationTypes.Alignment):
+                aligned_types = set()
+                for ann_id in [alignment.properties['target'], alignment.properties['source']]:
+                    if ':' in ann_id:
+                        view_id, ann_id = ann_id.split(':')
+                        aligned_types.add(str(self[view_id][ann_id].at_type))
+                    else:
+                        aligned_types.add(str(alignment_view[ann_id].at_type))
+                if str(at_type1) in aligned_types and str(at_type2) in aligned_types:
+                    alignments.append(alignment)
+            if len(alignments) > 0:
+                v_and_a[alignment_view.id] = alignments
+        return v_and_a
+
+    def get_views_for_document(self, doc_id: str):
+        """
+        Returns the list of all views that have annotations anchored on a particular document.
+        Note that when the document is insids a view (generated during the pipeline's running),
+        doc_id must be prefixed with the view_id.
+        """
+        views = []
+        for view in self.views:
+            annotations = view.get_annotations(document=doc_id)
+            try:
+                next(annotations)
+                views.append(view)
+            except StopIteration:
+                # search failed by the full doc_id string, now try trimming the view_id from the string and re-do the search
+                if ':' in doc_id:
+                    vid, did = doc_id.split(':')
+                    if view.id == vid:
+                        annotations = view.get_annotations(document=did)
+                        try:
+                            next(annotations)
+                            views.append(view)
+                        except StopIteration:
+                            # both search failed, give up and move to next view
+                            pass
+        return views
+
+
+    def get_all_views_contain(self, at_types: Union[ThingTypesBase, str, List[Union[str, ThingTypesBase]]]) -> List[View]:
+        """
+        Returns the list of all views in the MMIF if given types
+        are present in that view's 'contains' metadata.
+
+        :param at_types: a list of types or just a type to check for. When given more than one types, all types must be found.
         :return: the list of views that contain the type
         """
-        return [view for view in self.views if at_type in view.metadata.contains]
+        if isinstance(at_types, str) or isinstance(at_types, ThingTypesBase):
+            return [view for view in self.views if str(at_types) in view.metadata.contains]
+        else:
+            return [view for view in self.views
+                    if all(map(lambda x: str(x) in view.metadata.contains, at_types))]
 
-    def get_view_contains(self, at_type: str) -> Optional[View]:
+    def get_views_contain(self, at_types: Union[ThingTypesBase, str, List[Union[str, ThingTypesBase]]]) -> List[View]:
+        """
+        An alias to `get_all_views_contain` method.
+        """
+        return self.get_all_views_contain(at_types)
+
+    def get_view_contains(self, at_types: Union[ThingTypesBase, str, List[Union[str, ThingTypesBase]]]) -> Optional[View]:
         """
         Returns the last view appended that contains the given
-        type in its 'contains' metadata.
+        types in its 'contains' metadata.
 
-        :param at_type: the type to check for
+        :param at_types: a list of types or just a type to check for. When given more than one types, all types must be found.
         :return: the view, or None if the type is not found
         """
         # will return the *latest* view
         # works as of python 3.6+ (checked by setup.py) because dicts are deterministically ordered by insertion order
         for view in reversed(self.views):
-            if at_type in view.metadata.contains:
-                return view
+            if isinstance(at_types, str) or isinstance(at_types, ThingTypesBase):
+                if str(at_types) in view.metadata.contains:
+                    return view
+            else:
+                if all(map(lambda x: str(x) in view.metadata.contains, at_types)):
+                    return view
         return None
 
     def __getitem__(self, item: str) -> Union[Document, View, Annotation]:
@@ -306,6 +383,9 @@ class MmifMetadata(MmifObject):
     """
 
     def __init__(self, metadata_obj: Union[bytes, str, dict] = None) -> None:
+        # TODO (krim @ 10/7/20): there could be a better name and a better way to give a value to this
+        self.mmif: str = f"http://mmif.clams.ai/{mmif.__specver__}"
+        self._required_attributes = pvector(["mmif"])
         super().__init__(metadata_obj)
 
 
