@@ -69,7 +69,7 @@ class MmifObject(object):
     And also, a subclass that has one or more *named* attributes, it must
     set those attributes in the __init__() before calling super method. When
     serializing a MmifObject, all *empty* attributes will be ignored, so for
-    optional named attributes, you must leave leave the values empty, but
+    optional named attributes, you must leave the values empty (len == 0), but
     NOT None. Any None-valued named attributes will cause issues with current
     implementation.
 
@@ -110,11 +110,11 @@ class MmifObject(object):
     def set_additional_property(self, key: str, value: Any) -> None:
         """
         Method to set values in _unnamed_attributes.
+
         :param key: the attribute name
         :param value: the desired value
         :return: None
-        :raise: AttributeError if additional properties are disallowed
-                by :func:`disallow_additional_properties`
+        :raise: AttributeError if additional properties are disallowed by :func:`disallow_additional_properties`
         """
         if self._unnamed_attributes is None:
             raise AttributeError(f"Additional properties are disallowed by {self.__class__}")
