@@ -118,9 +118,7 @@ class View(FreezableMmifObject):
         Look for certain annotations in this view, specified by parameters
 
         :param at_type: @type of the annotations to look for. When this is None, any @type will match.
-        :param properties: properties of the annotations to look for.
-        When given more than one property, all properties must match.
-        Note that some annotation properties (annotation type medata) are specified in the `contains` view metadata
+        :param properties: properties of the annotations to look for. When given more than one property, all properties must match. Note that some annotation properties (annotation type medata) are specified in the `contains` view metadata
         """
         def prop_check(k, v, *props):
             return any(k in prop and prop[k] == v for prop in props)
@@ -181,6 +179,7 @@ class ViewMetadata(FreezableMmifObject):
         self.timestamp: Optional[datetime] = None
         self.app: str = ''
         self.contains: ContainsDict = ContainsDict()
+        self.parameters: dict = {}
         self._required_attributes = pvector(["app", "contains"])
         super().__init__(viewmetadata_obj)
 
@@ -237,6 +236,22 @@ class ViewMetadata(FreezableMmifObject):
             self.contains[final_key] = new_contain
             return new_contain
 
+    def add_parameters(self, param_dict: dict = None, **param_kwargs):
+        if param_dict is None:
+            self.parameters = {}
+        else:
+            self.parameters = param_dict
+        self.parameters.update(dict(param_kwargs))
+
+    def add_parameter(self, param_key, param_value):
+        self.parameters[param_key] = param_value
+
+    def get_parameter(self, param_key):
+        try:
+            return self.parameters[param_key]
+        except KeyError:
+            raise KeyError(f"parameter \"{param_key}\" is not set in the view: {self.serialize()}")
+
 
 class Contain(FreezableMmifObject):
     """
